"""

Utils python library.
"""
# __all__ = ["path"] #it is taken to be the list of module names that should be imported

# __version__ = "0.1.1"
__author__ = 'Andrew'

from aireplication.ultils.data import TimeSeriesGenerator
